#!/bin/sh

draw_main_menu () {
	clear
	echo "Neoware, Inc. - 2007 - USB Image Update and Retrieval Tool"
	echo "Version 2.2.2"
	echo ""
	echo "1.  Image Write Mode"
	echo "2.  Image Record Mode"
	echo "3.  Image Record Mode (XPe)"
	echo "----------------------"
	echo "4.  Secure Erase Flash"
	echo "5.  Format Flash ext2"
	echo "6.  Exit"
	echo ""
}

main_menu_selection () {
	valid_choice=0
	while [ $valid_choice -ne 1 ]; do
		echo -n "Select an image operation mode: "
		read user_selection
		case $user_selection in
			"1" )
				valid_choice=1
				write_image
				;;
			"2" )
				valid_choice=1
				xpe_image_record_mode=0
				pull_image
				;;
			"3" )
				valid_choice=1
				xpe_image_record_mode=1
				pull_image
				;;
			"4" )
				valid_choice=1
				zero_flash
				;;
			"5" )
				valid_choice=1
				format_flash
				;;
			"6" )
				valid_choice=1
				exit 0
				;;
			*   )
				draw_main_menu
				echo $user_selection is an invalid choice. Please choose again.
				;;
		esac
	done
}

get_flash_params () {
	if `/sbin/fdisk -l $flash_device 2>&1 | grep -q valid`; then
		echo "WARNING: Valid partition table not detected!!"
		echo "Defaulting to full disk record mode."
		recordEntireFlash="1"
	else 
		unit_size=`/sbin/fdisk -l $flash_device | grep bytes | grep -v Disk | awk '{print $9}'`
		end_unit=`/sbin/fdisk -l $flash_device | grep $flash_device | tail -1 | sed -e 's/*//' | awk '{print $3}'`
	fi
}

unix_friendly () {
	inFile="$1"
	cat $inFile | /usr/bin/tr -d '\r' > /tmp/tmpfile_unix_friendly
	mv /tmp/tmpfile_unix_friendly $inFile
}

source_settings () {
	cp "$USBKEY/settings.ini" /tmp/settings.ini
	unix_friendly "/tmp/settings.ini"
	. /tmp/settings.ini
}

zero_flash () {
	if [ $enableZeroDOM = "1" ]; then
		echo ""
		echo WARNING: The image on \"$flash_device\" is about to be overwritten.
		echo -n "Do you wish to proceed with the operation? [yes/no] "
		read result
		case $result in
			"yes" | "Yes" | "YES" | "y" | "Y" )
				echo "\"Secure Erasing\" $flash_device"...
				( while true ; do echo -n "."; sleep 1; done ) &
				dd if=$secure_write_device of=$flash_device bs=1M > /dev/null 2>&1
				sync
				kill -9 $! > /dev/null 2>&1
				clear; clear; clear
				echo ""
				echo "\"Secure Erase\" operation complete."
				echo ""
				;;
			* )
				echo "\"Secure Erase\" operation aborted."
				;;
		esac
	else
		draw_main_menu
		echo "\"Secure Erase\" mode is disabled"
		echo ""
		main_menu_selection
	fi
}

format_flash () {
	if [ $enableFormatDOM = "1" ]; then
		echo ""
		echo WARNING: The image on \"$flash_device\" is about to be overwritten.
		echo -n "Do you wish to proceed with the operation? [yes/no] "
		read result
		case $result in
			"yes" | "Yes" | "YES" | "y" | "Y" )
				echo "Partitioning flash device $flash_device"...
				/sbin/fdisk $flash_device > /dev/null 2>&1 << EOF
o
w
EOF
				/sbin/fdisk $flash_device > /dev/null 2>&1 << EOF
n
p
1


w
EOF
				echo "Formatting flash device $flash_device"...
				/sbin/mke2fs "$flash_device"1 > /dev/null 2>&1
				/sbin/fdisk -l $flash_device
				echo ""
				echo "\"Format Flash ext2\" operation complete."
				;;
			* )
				echo "\"Format Flash ext2\" operation aborted."
				;;
		esac
	else
		draw_main_menu
		echo "\"Format Flash ext2\" mode is disabled"
		echo ""
		main_menu_selection
	fi
}

create_image_list () {
	ls *.dd* | grep -v "image.dd.goes.here" > /tmp/image_list.tmp
}

display_image_list () {
	echo ""
	echo "[Num.]    [Image Name]"
	image_index=1
	cat /tmp/image_list.tmp | {
	while read image_name
	do
		echo $image_index "       " $image_name
		image_index=$(($image_index + 1))
	done
	}
}

get_numeric_input () {
	while true; do
		read user_input
		case $user_input in
			*[!0-9]* | "" )
				echo "Only numeric values are allowed. Please choose again."
				;;
			* )
				break
				;;
		esac
	done
}
				

get_image_filename () {
	valid_choice=0
	while [ $valid_choice -ne 1 ]; do
		echo ""
		echo -n "Enter the desired image number and press enter to program: "
		get_numeric_input
		user_selection=$user_input
		if [ $user_selection -gt 0 ] && [ $user_selection -le `grep -c "" /tmp/image_list.tmp` ]; then
			valid_choice=1
			image_index=1
			image_filename=`cat /tmp/image_list.tmp | {
			while read image_name
			do
				if [ $image_index = $user_selection ]; then
					echo $image_name
					break
				else
					image_index=$(($image_index + 1))
				fi
			done
			}`
		else
			clear
			echo $user_selection is an invalid choice. Please choose again.
			display_image_list
		fi
	done
}

write_image () {
	if [ $enableWriteMode = "1" ]; then
		if [ ! $1 = "" ]; then
			image_filename=$1
		else
			create_image_list
			display_image_list
			get_image_filename
		fi
		if [ $confirmWrites = "1" ]; then
			echo ""
			echo WARNING: The image on \"$flash_device\" is about to be overwritten.
			echo -n "Do you wish to proceed with the update? [yes/no] "
			read result
		else
			result="yes"
		fi
		case $result in
			"yes" | "Yes" | "YES" | "y" | "Y" )
				echo Writing image $image_filename to $flash_device...
				if `real_od -N 2 -t x1 $image_filename | grep -q "1f 8b"`; then
					( while true ; do echo -n "."; sleep 1; done ) & 
					gunzip -c $image_filename | dd of=$flash_device bs=4096 > /tmp/dd_log.tmp 2>&1
				elif `real_od -N 2 -t x1 $image_filename | grep -q "50 4b"`; then
					( while true ; do echo -n "."; sleep 1; done ) & 
					unzip -p $image_filename | dd of=$flash_device bs=4096 > /tmp/dd_log.tmp 2>&1
				else
					( while true ; do echo -n "."; sleep 1; done ) &
					dd if=$image_filename of=$flash_device bs=4096 > /tmp/dd_log.tmp 2>&1
				fi
				sync
				kill -9 $! > /dev/null 2>&1
				clear; clear; clear
				cat /tmp/dd_log.tmp
				echo ""
				echo Image operation complete.
				echo ""
				;;
			* )
				echo Image was not overwritten.
				;;
		esac
	else
		draw_main_menu
		echo "\"Image Write\" mode is disabled"
		echo ""
		main_menu_selection
	fi
}

pull_image () {
	if [ $enableRecordMode = "1" ]; then
		get_flash_params
		if [ $xpe_image_record_mode = "1" ]; then
			mkdir /tmp/msdos
			mount -t vfat "$flash_device"1 /tmp/msdos
			if [ ! -f /tmp/msdos/mac_name.exe ]; then
				clear
				echo "ERROR: XPe image has not been pre-processed."
				echo "Please reference the included documentation for instructions"
				echo "on how to properly prepare an XPe image for recording."
				echo ""
				echo "Image record operation aborted..."
				echo ""
				break
			fi
			umount /tmp/msdos
		fi
		echo ""
		echo "Please enter a name for the image that will be pulled (e.g. my_image)."
		echo "Do not include the file extension. This will be added automatically."
		echo ""
		echo "These are the existing image file names:"
		echo "----------------------------------------"
		create_image_list; cat /tmp/image_list.tmp
		echo ""
		echo -n "File name: "
		read user_file_name
		echo -n "Gzip compress image? [yes/no] "
		read user_gzip
		case $user_gzip in
			"yes" | "Yes" | "YES" | "y" | "Y" ) 
				user_file_name="$user_file_name".dd.gz
				if [ -f "$user_file_name" ]; then
					echo $user_file_name already exists.
					echo Image was not recorded.
					break
				fi
				echo ""
				echo "Recording compressed image $user_file_name from $flash_device..."
				if [ $recordEntireFlash = "1" ]; then
					( while true ; do echo -n "."; sleep 1; done ) &
					( dd if=$flash_device bs=4096 | gzip - > $user_file_name ) > /tmp/dd_log.tmp 2>&1
				else
					( while true ; do echo -n "."; sleep 1; done ) &
					( dd if=$flash_device bs=$unit_size count=$end_unit | gzip - > $user_file_name ) > /tmp/dd_log.tmp 2>&1
				fi
				sync
				;;
			* )
				user_file_name="$user_file_name".dd
				if [ -f "$user_file_name" ]; then
					echo $user_file_name already exists.
					echo Image was not recorded.
					break
				fi
				echo ""
				echo "Recording image $user_file_name from $flash_device..."
				if [ $recordEntireFlash = "1" ]; then
					( while true ; do echo -n "."; sleep 1; done ) &
					( dd if=$flash_device of=$user_file_name bs=4096 ) > /tmp/dd_log.tmp 2>&1
				else
					( while true ; do echo -n "."; sleep 1; done ) &
					( dd if=$flash_device of=$user_file_name bs=$unit_size count=$end_unit ) > /tmp/dd_log.tmp 2>&1
				fi
				sync
				;;
		esac
		kill -9 $! > /dev/null 2>&1
		clear; clear; clear
		cat /tmp/dd_log.tmp
		echo ""
		echo Image operation complete.
		echo ""
	else
		draw_main_menu
		echo "\"Image Record\" mode is disabled"
		echo ""
		main_menu_selection
	fi
}

wait_for_keyboard () {
	iii=0
	while [ $iii -lt 8 ]; do
		grep -i keyboard /var/log/messages > /dev/null 2>&1
		if [ $? -eq 0 ]; then
			break
		else
			echo Waiting for keyboard. Please standby...
			iii=$((iii + 1))
			sleep 2
		fi
	done
}

wait_for_keyboard
source_settings


cd $USBKEY/$images_dir

if [ $useQuickImageMode -eq 1 ] && [ ! $quickImage = "" ]; then
	if [ -f $quickImage ]; then
		write_image $quickImage
	else
		echo "ERROR: $quickImage not found. Defaulting to normal mode."
		echo "Press enter to continue..."
		read
		draw_main_menu
		main_menu_selection
	fi
else
	draw_main_menu
	main_menu_selection
fi

exit 0
